@@ -11,7 +11,7 @@
 **  
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall.
+**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall.
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -48,15 +48,37 @@
 
 #include "iselect_global.h"
 #include "iselect_browse.h"
-
+#include "iselect_keys.h"
 
 extern char *iSelect_Help[];
 extern char *iSelect_README[];
 
+void strip(char *string);
+void diehard(void);
+void diesoft(void);
+void setup(void);
+char *iSelect_InputField(int wYSize, int wXSize, int wYPos, int wXPos, int bAllowEmpty);
+void iSelect_Draw(WINDOW *wField,
+                  int wYSize, int wXSize, int wYPos, int wXPos,
+                  int nAbsFirstLine, int nAbsLastLine,
+                  int nRelMarked,
+                  int nRelFirstDraw, int nRelLastDraw,
+                  int nLines,
+                  WINDOW *sField, char *title, char *name,
+                  WINDOW *mField, char *msg, char *tagbegin, char *tagend);
+int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos, int multiselect,
+                    int sYSize, int sXSize, int sYPos, int sXPos, char *title, char *name,
+                    int mYSize, int mXSize, int mYPos, int mXPos,
+                    char **keystr, char *tagbegin, char *tagend);
+int iSelect(char *caBuf, int pos, char *title, char *name,
+            char *tagbegin, char *tagend, int stripco, int stripws, 
+            int browsealways, int allselectable,
+            int multiselect, int exitnoselect,
+            char **keystr); 
+
 /*
  *  GLOBALS
  */
-char         caBuf[MAXBUF];       /* filled by main() */
 struct Line *spaLines[MAXLINES];  /* filled by iSelect() */
 int          nLines;
 
@@ -122,6 +144,109 @@ void setup(void)
     return;
 }
 
+/*
+ *  Function to query an input string
+ */
+char *iSelect_InputField(int wYSize, int wXSize, int wYPos, int wXPos, int bAllowEmpty)
+{
+    WINDOW *wField;
+    int c;
+    int nBufEnd;
+    int nBufCurPos;
+    int nScrCurPos;
+    int bEOI;
+    char caBuf[1024];
+    char *cpBuf;
+    int i;
+
+    cpBuf = caBuf;
+
+    wField = newwin(wYSize, wXSize, wYPos, wXPos);
+    keypad(wField, TRUE);
+    scrollok(wField, FALSE);
+
+    nScrCurPos = 0;
+    nBufCurPos = 0;
+    nBufEnd = 0;
+    bEOI = FALSE;
+
+    wmove(wField, 0, nScrCurPos);
+    while (bEOI == FALSE) {
+        c = wgetch(wField);
+        if (c >= KEY_MIN && c <= KEY_MAX) {
+            /* NOP */
+        }
+        else {
+            c = c & 0xff; /* strip down to 8bit */
+            if (c < 32 || c > 126) { 
+                /*
+                 *  a control code
+                 */
+                if (c == '\n') {      /* RETURN */
+                    if (nBufEnd == 0 && !bAllowEmpty) 
+                        continue;
+                    cpBuf[nBufEnd] = '\0';
+                    bEOI = TRUE;
+                }
+                else if (c == 0x1b) { /* ESCAPE */
+                    strcpy(caBuf, "ESC");
+                    bEOI = TRUE;
+                }
+                else if (c == 0x04) { /* DELETE */
+                    waddch(wField, '?');
+                    wrefresh(wField);
+                }
+                else if (c == 0x7f) { /* BACKSPACE */
+                    if (nBufCurPos != 0) {
+                        for (i = nBufCurPos; i < nBufEnd; i++) {
+                            cpBuf[i-1] = cpBuf[i];
+                        }
+                        cpBuf[i] = '\0';
+                        nBufEnd--;
+                        nBufCurPos--;
+
+                        if (nScrCurPos == (wXSize-1) && nBufEnd >= (wXSize-1)) {
+                            wmove(wField, 0, 0);
+                            for (i = nBufEnd-(wXSize-1); i < nBufEnd; i++) {
+                                waddch(wField, cpBuf[i]);
+                            }
+                        }
+                        else {
+                            nScrCurPos--;
+                            wmove(wField, 0, nScrCurPos);
+                            waddch(wField, ' ');
+                            wmove(wField, 0, nScrCurPos);
+                        }
+                        wrefresh(wField);
+                    }
+                }
+            }
+            if (c >= 32 && c <= 126) { 
+                /*
+                 *  a printable character
+                 */
+                if (nScrCurPos == (wXSize-1)) {
+                    wmove(wField, 0, 0);
+                    for (i = nBufEnd-(wXSize-1)+1; i < nBufEnd; i++) {
+                        waddch(wField, cpBuf[i]);
+                    }
+                    nScrCurPos--; /* neutralize following increment */
+                }
+                cpBuf[nBufEnd++] = c;
+                nBufCurPos++;
+                nScrCurPos++;
+                waddch(wField, c);
+                wrefresh(wField);
+            }
+        }
+    }
+
+    fflush(stdin);
+    delwin(wField);
+
+    return strdup(caBuf);
+}
+
 /*
  *  Function to draw a complete screen
  */
@@ -132,11 +257,23 @@ void iSelect_Draw(WINDOW *wField,
                   int nRelFirstDraw, int nRelLastDraw,
                   int nLines,
                   WINDOW *sField, char *title, char *name,
-                  WINDOW *mField, char *msg)
+                  WINDOW *mField, char *msg, char *tagbegin, char *tagend)
 {
-    int i, j, mode;
+    int i, j, k, mode;
     int percent;
     char ca[80];
+    char boldbegin[1024];
+    int nboldbegin;
+    char boldend[1024];
+    int nboldend;
+
+    /*
+     *  create actual bold tags
+     */
+    sprintf(boldbegin, "%sb%s", tagbegin, tagend);
+    nboldbegin = strlen(boldbegin);
+    sprintf(boldend, "%s/b%s", tagbegin, tagend);
+    nboldend = strlen(boldend);
 
     /*
      *  draw browser window 
@@ -158,9 +295,24 @@ void iSelect_Draw(WINDOW *wField,
             waddstr(wField, "*");
         else
             waddstr(wField, " "); /* cursor pos == first blank */
-        for (j = 0; spaLines[nAbsFirstLine+nRelFirstDraw+i]->cpLine[j] != NUL && j <= (wXSize-1)-2; j++)
+        for (j = k = 0; spaLines[nAbsFirstLine+nRelFirstDraw+i]->cpLine[j] != NUL && j <= (wXSize-1)-2;) {
+            if (strniEQ(&spaLines[nAbsFirstLine+nRelFirstDraw+i]->cpLine[j], boldbegin, nboldbegin)) {
+                mode = mode | A_BOLD;
+                wattrset(wField, mode);
+                j += nboldbegin;
+                continue;
+            }
+            if (strniEQ(&spaLines[nAbsFirstLine+nRelFirstDraw+i]->cpLine[j], boldend, nboldend)) {
+                mode = mode & ~A_BOLD;
+                wattrset(wField, mode);
+                j += nboldend;
+                continue;
+            }
             waddch(wField, spaLines[nAbsFirstLine+nRelFirstDraw+i]->cpLine[j]);
-        for ( ; j <= (wXSize-1)-2; j++)
+            j++;
+            k++;
+        }
+        for ( ; k <= (wXSize-1)-2; k++)
             waddch(wField, ' ');
         wattrset(wField, A_NORMAL);
     }
@@ -209,8 +361,8 @@ void iSelect_Draw(WINDOW *wField,
  */
 int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos, int multiselect,
                     int sYSize, int sXSize, int sYPos, int sXPos, char *title, char *name,
-                    int mYSize, int mXSize, int mYPos, int mXPos
-                    )
+                    int mYSize, int mXSize, int mYPos, int mXPos,
+                    char **keystr, char *tagbegin, char *tagend)
 {
     WINDOW *wField;
     WINDOW *sField;
@@ -228,10 +380,13 @@ int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos,
     int x;
     char msg[1024];
     char ca[1024];
+    char ca3[1024];
     char *cp;
     char *cp2;
+    char *cp3;
     char **cpp;
     int ok;
+    int bAllowEmpty;
 
     /*
      *  Browser field
@@ -325,10 +480,13 @@ int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos,
                       nRelFirstDraw, nRelLastDraw,
                       nLines,
                       sField, title, name,
-                      mField, msg);
+                      mField, msg,
+                      tagbegin, tagend);
         wrefresh(wField);
         strcpy(msg, "");
         c = wgetch(wField);
+        *keystr = key2asc(c);
+        c = do_custom_key(c);
         if (c == KEY_LEFT)
             c = 'q';
         if (c == KEY_RIGHT)
@@ -441,6 +599,49 @@ int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos,
                             strcpy(msg, "Line not selectable.");
                         }
                     }
+
+                    /* additionally ask for query strings */
+                    if (bEOI == TRUE) {
+                        cp = spaLines[nAbsFirstLine+nRelMarked]->cpResult;
+                        cp2 = ca;
+                        while (bEOI == TRUE && *cp != NUL) {
+                            if (strnEQ(cp, "%[", 2)) {
+                                cp += 2;
+                                for (cp3 = cp; !strniEQ(cp3, "]s", 2); cp3++)
+                                    ;
+                                strncpy(ca3, cp, cp3-cp);
+                                ca3[cp3-cp] = NUL;
+                                cp = cp3+1;
+                                if (*cp == 's')
+                                    bAllowEmpty = TRUE;
+                                else
+                                    bAllowEmpty = FALSE;
+                                cp++;
+    
+                                sprintf(msg, "%s: ", ca3);
+                                iSelect_Draw(wField, wYSize, wXSize, wYPos, wXPos, nAbsFirstLine, nAbsLastLine, -1, nRelFirstDraw, nRelLastDraw, nLines, sField, title, name, mField, msg, tagbegin, tagend);
+                                wrefresh(wField);
+                                cp3 = iSelect_InputField(mYSize, mXSize-strlen(msg), mYPos, mXPos+strlen(msg), bAllowEmpty);
+                                if (strEQ(cp3, "ESC")) {
+                                    bEOI = FALSE;
+                                    spaLines[nAbsFirstLine+nRelMarked]->fSelected = FALSE;
+                                    strcpy(msg, "Selection cancelled.");
+                                    continue;
+                                }
+                                strcpy(msg, "");
+                                strcpy(cp2, cp3);
+                                cp2 += strlen(cp3);
+                            }
+                            else {
+                                *cp2++ = *cp++;
+                            }
+                        }
+                        if (bEOI == TRUE) {
+                            *cp2 = NUL;
+                            if (strNE(spaLines[nAbsFirstLine+nRelMarked]->cpResult, ca))
+                                spaLines[nAbsFirstLine+nRelMarked]->cpResult = strdup(ca);
+                        }
+                    }
                 }
             }
             if (c >= 32 && c <= 126) { 
@@ -514,7 +715,7 @@ int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos,
                          strcpy(msg, "Help Page: Press 'q' to exit");
                      else 
                          strcpy(msg, "Version Page: Press 'q' to exit");
-                     iSelect_Draw(wField, wYSize, wXSize, wYPos, wXPos, nAbsFirstLine, nAbsLastLine, nRelMarked, nRelFirstDraw, nRelLastDraw, nLines, sField, title, name, mField, msg);
+                     iSelect_Draw(wField, wYSize, wXSize, wYPos, wXPos, nAbsFirstLine, nAbsLastLine, nRelMarked, nRelFirstDraw, nRelLastDraw, nLines, sField, title, name, mField, msg, tagbegin, tagend);
                      wrefresh(wField);
 
                      hField = newwin(wYSize, wXSize, wYPos, wXPos);
@@ -557,7 +758,7 @@ int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos,
                      nRelFirstDraw = 0;
                      nRelLastDraw = nAbsLastLine-nAbsFirstLine;
                      strcpy(msg, "");
-                     iSelect_Draw(wField, wYSize, wXSize, wYPos, wXPos, nAbsFirstLine, nAbsLastLine, nRelMarked, nRelFirstDraw, nRelLastDraw, nLines, sField, title, name, mField, msg);
+                     iSelect_Draw(wField, wYSize, wXSize, wYPos, wXPos, nAbsFirstLine, nAbsLastLine, nRelMarked, nRelFirstDraw, nRelLastDraw, nLines, sField, title, name, mField, msg, tagbegin, tagend);
 #ifndef USE_SLCURSES
                      redrawwin(wField);
 #endif
@@ -586,14 +787,16 @@ int iSelect_Browser(int wYSize, int wXSize, int wYPos, int wXPos, int selectpos,
 /*
  *  The iSelect function...
  */
-int iSelect(int pos, char *title, char *name,
-            int stripco, int stripws, 
+int iSelect(char *caBuf, int pos, char *title, char *name,
+            char *tagbegin, char *tagend, int stripco, int stripws, 
             int browsealways, int allselectable,
-            int multiselect, int exitnoselect) 
+            int multiselect, int exitnoselect,
+            char **keystr) 
 {
     WINDOW *wMain;
     int i, j, k;
     char ca[MAXLINELEN], ca2[MAXLINELEN];
+    char ca3[MAXLINELEN];
     char *cp, *cps, *cpe;
     struct Line *spLine;
     int rc;
@@ -622,23 +825,26 @@ int iSelect(int pos, char *title, char *name,
             }
 
             spLine = (struct Line *)malloc(sizeof(struct Line));
-            if ((cp = strstr(ca, "%%S")) != NULL) {
+
+            sprintf(ca2, "%ss", tagbegin);
+            sprintf(ca3, "%sS", tagbegin);
+            if (   (cp = strstr(ca, ca2)) != NULL
+                || (cp = strstr(ca, ca3)) != NULL) {
                 /* is a selectable line */
                 cps = cp;
-                cp += 3;
+                cp += strlen(tagbegin)+1;
                 if (strnEQ(cp, ":", 1)) {
                     cp++;
-                    for (k = 0; *cp != '%'; k++) {
+                    for (k = 0; !strnEQ(cp, tagend, strlen(tagend)); k++)
                         ca2[k] = *cp++;
-                    }
                     ca2[k] = NUL;
-                    cpe = cp+2;
+                    cpe = cp+strlen(tagend);
                     for (; *cpe != NUL; )
                         *cps++ = *cpe++;
                     *cps = NUL;
                 }
-                else if (strnEQ(cp, "%%", 2)) {
-                    cpe = cp+2;
+                else if (strnEQ(cp, tagend, strlen(tagend))) {
+                    cpe = cp+strlen(tagend);
                     for (; *cpe != NUL; )
                         *cps++ = *cpe++;
                     *cps = NUL;
@@ -660,6 +866,7 @@ int iSelect(int pos, char *title, char *name,
                     spLine->fSelectable = FALSE;
                 spLine->cpResult = strdup(ca);
             }
+
             spLine->fSelected = FALSE;
             if (stripws)
                 strip(spLine->cpResult);
@@ -709,7 +916,9 @@ int iSelect(int pos, char *title, char *name,
      */
     rc = iSelect_Browser(/* Browser: */ LINES-2, COLS-2, 0, 1, pos, multiselect,
                          /* Status:  */ 1, COLS, LINES-2, 0, title, name,
-                         /* Message: */ 1, COLS-1, LINES-1, 0 );
+                         /* Message: */ 1, COLS-1, LINES-1, 0,
+                         /* Result:  */ keystr,
+                         /* Tags:    */ tagbegin, tagend);
 
     /*  
      *  delete the main window and
