@@ -29,7 +29,23 @@
     Santa Barbara, CA
     93101  */
 
+/*
+################### CAUTION ##################################
+# This file has been modified for inclusion into WML
+# D. Barbier   16/01/1999
+##############################################################
+*/
+
 #define LANGUAGE_DEFINITIONS_FILE 1
+#define MHTML_ARITHMETIC 1
+
+#if defined (_GNUC_) && !defined (VOLATILE)
+#  define VOLATILE volatile
+#endif
+
+#if !defined (VOLATILE)
+#  define VOLATILE
+#endif
 
 #include "mhtmlstd.h"
 #include "forms.h"
