@@ -2,6 +2,13 @@
 
 /*  Copyright (c) 1996 Brian J. Fox
     Author: Brian J. Fox (bfox@ai.mit.edu) Fri Jan  3 15:32:04 1997.  */
+/*
+################### CAUTION ##################################
+# This file has been modified for inclusion into WML
+# D. Barbier   16/01/1999
+##############################################################
+*/
+
 #if !defined (_MHTMLSTD_H_)
 #define _MHTMLSTD_H_
 
@@ -14,7 +21,12 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+
+#if defined (__linux__)
+#  define _NO_CTYPE 1
+#endif
 #include <ctype.h>
+
 #include <sys/types.h>
 #include <errno.h>
 #if defined (HAVE_SYS_FILE_H)
@@ -39,11 +51,13 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#if 0
 #if defined (__WINNT__)		/* CYGWIN defines sig_t in netdb.h. */
 #  if !defined (HAVE_TYPE_SIG_T)
 #    define HAVE_TYPE_SIG_T 1
 #  endif
 #endif
+#endif
 
 #include <sys/wait.h>
 
@@ -106,7 +120,11 @@
 #include <signal.h>
 
 #include <setjmp.h>
-#include <regex/regex.h>
+#if !defined (HAVE_REGEX_H)
+#  include <regex/regex.h>
+#else
+#  include <regex.h>
+#endif
 #include <bprintf/bprintf.h>
 #include <xmalloc/xmalloc.h>
 #include <wisper/wisp.h>
@@ -116,8 +134,12 @@
 #endif
 
 #if !defined (HAVE_TYPE_SIG_T)
+#if defined (__cplusplus)
+extern "C" typedef RETSIGTYPE (*sig_t) (int);
+#else
 typedef RETSIGTYPE (*sig_t) (int);
-#endif
+#endif /* CPLUSPLUS */
+#endif /* HAVE_TYPE_SIG_T */
 
 #if !defined (HAVE_TYPE_TIME_T)
 typedef long time_t;
