@@ -1053,11 +1053,7 @@ int main(int argc, char **argv, char **env)
     /* perl_destruct_level = 1; */
 
     /*  initialise the Perl Locale environment  */
-#if AC_perl_vnum < 500400
-    perl_init_i18nl14n(1); /* Perl 5.003 or lower */
-#else
-    perl_init_i18nl10n(1); /* Perl 5.004 or higher */
-#endif
+    perl_init_i18nl10n(1); /* Macro defined in eperl_perl5.h */
 
     /*  create command line...  */
     myargc = 0;
