@@ -37,6 +37,7 @@
 #include "eperl_global.h"
 #include "eperl_proto.h"
 
+char * getenv ();
 
 /*
 **
@@ -77,8 +78,8 @@ char **mysetenv(char **env, char *var, char *str, ...)
     /*  add the new entry if not replaced */
     if (!replaced) {
         envN[i++] = cp;
-        envN[i++] = NULL;
     }
+    envN[i] = NULL;
 
     /*  set the libc/exec variable which Perl uses */
     if (stillcalled) 
@@ -205,10 +206,14 @@ static int mytmpfilecnt = 0;
 char *mytmpfile(char *id)
 {
     char ca[1024];
-    char *cp;
+    char *cp, *tmpdir;
     int i;
 
-    sprintf(ca, "/tmp/%s.%d.tmp%d", id, (int)getpid(), mytmpfilecnt++);
+    tmpdir = getenv ("TMPDIR");
+    if (tmpdir == (char *) NULL)
+        tmpdir="/tmp";
+
+    sprintf(ca, "%s/%s.%d.tmp%d", tmpdir, id, (int)getpid(), mytmpfilecnt++);
     cp = strdup(ca);
     for (i = 0; mytmpfiles[i] != NULL; i++)
         ;
@@ -239,8 +244,8 @@ char *isotime(time_t *t)
     char timestr[128];
 
     tm = localtime(t);
-    sprintf(timestr, "%02d-%02d-19%02d %02d:%02d",
-                      tm->tm_mday, tm->tm_mon+1, tm->tm_year,
+    sprintf(timestr, "%02d-%02d-%04d %02d:%02d",
+                      tm->tm_mday, tm->tm_mon+1, tm->tm_year+1900,
                       tm->tm_hour, tm->tm_min);
     return strdup(timestr);
 }
@@ -257,13 +262,14 @@ char *ePerl_ReadSourceFile(char *filename, char **cpBufC, int *nBufC)
     FILE *fp = NULL;
     char *cpBuf = NULL;
     int nBuf;
-    char tmpfile[256];
+    char tmpfile[256], *ptr_tmpfile;
     int usetmp = 0;
     int c;
 
     if (stringEQ(filename, "-")) {
         /* file is given on stdin */
-        sprintf(tmpfile, "/tmp/eperl.tmp.%d", (int)getpid());
+        ptr_tmpfile = mytmpfile("ePerl.source");
+	sprintf(tmpfile, "%s", ptr_tmpfile);
         if ((fp = fopen(tmpfile, "w")) == NULL) {
             ePerl_SetError("Cannot open temporary source file %s for writing", tmpfile);
             CU(NULL);
